@@ -1,5 +1,6 @@
 """Base classes and function for REST, WMS, and WMF services."""
-from dataclasses import dataclass
+import logging
+import sys
 from typing import Dict, List, Optional, Tuple, Union
 
 import cytoolz as tlz
@@ -7,11 +8,18 @@ import pyproj
 from orjson import JSONDecodeError
 from owslib.wfs import WebFeatureService
 from owslib.wms import WebMapService
+from shapely.geometry import MultiPoint, Point, Polygon
 
 from . import utils
 from .exceptions import InvalidInputType, InvalidInputValue, MissingInputs, ServerError, ZeroMatched
 from .utils import RetrySession
 
+logger = logging.getLogger(__name__)
+logger.setLevel(logging.INFO)
+handler = logging.StreamHandler(sys.stdout)
+handler.setFormatter(logging.Formatter(""))
+logger.handlers = [handler]
+logger.propagate = False
 DEF_CRS = "epsg:4326"
 
 
@@ -53,37 +61,44 @@ class ArcGISRESTfulBase:
 
         self.session = RetrySession()
         self.base_url = base_url[:-1] if base_url[-1] == "/" else base_url
-        self.test_url()
 
-        self._layer = ""
+        self._layer = 99
         self._outformat = outformat
         self._spatial_relation = spatial_relation
         self._outfields = outfields if isinstance(outfields, list) else [outfields]
         self._n_threads = n_threads
         self.nfeatures = 0
         self.crs = crs
-        self.out_sr = pyproj.CRS(self.crs).to_epsg()
+        out_sr = pyproj.CRS(self.crs).to_epsg()
+        if out_sr is None:
+            raise InvalidInputType("crs", "a valid CRS")
+        self.out_sr = out_sr
         self.valid_layers = self.get_validlayers()
+        self.extent: Optional[Tuple[float, float, float, float]] = None
+        self.feature_types: Optional[Dict[int, str]] = None
 
         self._zeromatched = "No feature ID were found within the requested region."
+        self.test_url()
 
     @property
-    def layer(self) -> str:
+    def layer(self) -> int:
+        """Set service layer."""
         return self._layer
 
     @layer.setter
     def layer(self, value: int) -> None:
+        """Set service layer."""
         try:
             existing_lyr = int(self.base_url.split("/")[-1])
             self.base_url = self.base_url.replace(f"/{existing_lyr}", "")
         except ValueError:
             pass
 
-        if value not in self.valid_layers:
+        if f"{value}" not in self.valid_layers:
             valids = [f'"{i}" for {n}' for i, n in self.valid_layers.items()]
             raise InvalidInputValue("layer", valids)
 
-        self._layer = f"{value}"
+        self._layer = value
         try:
             existing_lyr = int(self.base_url.split("/")[-1])
             self.base_url = self.base_url.replace(f"/{existing_lyr}", f"/{value}")
@@ -94,10 +109,12 @@ class ArcGISRESTfulBase:
 
     @property
     def outformat(self) -> str:
+        """Set service output format."""
         return self._outformat
 
     @outformat.setter
     def outformat(self, value: str) -> None:
+        """Set service output format."""
         if self.base_url is not None and value.lower() not in self.query_formats:
             raise InvalidInputValue("outformat", self.query_formats)
 
@@ -105,10 +122,12 @@ class ArcGISRESTfulBase:
 
     @property
     def spatial_relation(self) -> str:
+        """Set spatial relationship of the input geometry with the source data."""
         return self._spatial_relation
 
     @spatial_relation.setter
     def spatial_relation(self, value: str) -> None:
+        """Set spatial relationship of the input geometry with the source data."""
         valid_spatialrels = [
             "esriSpatialRelIntersects",
             "esriSpatialRelContains",
@@ -126,10 +145,12 @@ class ArcGISRESTfulBase:
 
     @property
     def outfields(self) -> List[str]:
+        """Set service output field(s)."""
         return self._outfields
 
     @outfields.setter
     def outfields(self, value: Union[List[str], str]) -> None:
+        """Set service output field(s)."""
         if not isinstance(value, (list, str)):
             raise InvalidInputType("outfields", "str or list")
 
@@ -137,20 +158,24 @@ class ArcGISRESTfulBase:
 
     @property
     def n_threads(self) -> int:
+        """Set number of threads."""
         return self._n_threads
 
     @n_threads.setter
     def n_threads(self, value: int) -> None:
+        """Set number of threads."""
         if not isinstance(value, int) or value < 0:
             raise InvalidInputType("n_threads", "positive int")
         self._n_threads = value
 
     @property
     def max_nrecords(self) -> int:
+        """Set maximum number of features per request."""
         return self._max_nrecords
 
     @max_nrecords.setter
     def max_nrecords(self, value: int) -> None:
+        """Set maximum number of features per request."""
         if value > self.max_nrecords:
             raise ValueError(
                 f"The server doesn't accept more than {self.max_nrecords}" + " records per request."
@@ -162,10 +187,15 @@ class ArcGISRESTfulBase:
 
     @property
     def featureids(self) -> List[Tuple[str, ...]]:
+        """Set feature ID(s)."""
         return self._featureids
 
     @featureids.setter
-    def featureids(self, value: Union[List[int], int]) -> None:
+    def featureids(self, value: Union[List[int], int, None]) -> None:
+        """Set feature ID(s)."""
+        if value is None:
+            raise ZeroMatched(self._zeromatched)
+
         if not isinstance(value, (list, int)):
             raise InvalidInputType("featureids", "int or list")
 
@@ -175,9 +205,11 @@ class ArcGISRESTfulBase:
         if self.nfeatures == 0:
             raise ZeroMatched(self._zeromatched)
 
+        logger.info(f"{self.nfeatures:,} features found in the requested region.")
         self._featureids = list(tlz.partition_all(self.max_nrecords, oids))
 
     def get_validlayers(self) -> Dict[str, str]:
+        """Get all the valid service layer."""
         try:
             existing_lyr = int(self.base_url.split("/")[-1])
             url = self.base_url.replace(f"/{existing_lyr}", "")
@@ -189,13 +221,14 @@ class ArcGISRESTfulBase:
 
         layers = {"No layer": ""}
         try:
-            layers = {lyr["id"]: lyr["name"] for lyr in resp.json()["layers"]}
+            layers = {f"{lyr['id']}": lyr["name"] for lyr in resp.json()["layers"]}
         except (JSONDecodeError, KeyError):
             raise ZeroMatched(f"The requested layer does not exists in:\n{self.base_url}")
 
         return layers
 
     def get_validfields(self) -> Dict[str, str]:
+        """Get all the valid service output fields."""
         resp = self.session.get(self.base_url, {"f": "json"}).json()
         return {f["name"]: f["type"].replace("esriFieldType", "").lower() for f in resp["fields"]}
 
@@ -223,25 +256,61 @@ class ArcGISRESTfulBase:
                 self.extent = utils.MatchCRS.bounds(bounds, crs, DEF_CRS)
             except KeyError:
                 self.extent = None
+            try:
+                self.feature_types = dict(
+                    zip((tlz.pluck("id", resp["types"])), tlz.pluck("name", resp["types"]))
+                )
+            except KeyError:
+                self.feature_types = None
         except KeyError:
             raise ServerError(self.base_url)
 
+    def _esri_query(
+        self,
+        geom: Union[
+            Polygon,
+            Point,
+            MultiPoint,
+            Tuple[float, float, float, float],
+        ],
+        geo_crs: str = DEF_CRS,
+    ) -> Dict[str, Union[str, bytes]]:
+        """Generate geometry queries based on ESRI template."""
+        if isinstance(geom, tuple) and len(geom) == 4:
+            geom = utils.MatchCRS.bounds(geom, geo_crs, self.crs)  # type: ignore
+            return utils.ESRIGeomQuery(geom, self.out_sr).bbox()
+
+        if isinstance(geom, Point):
+            geom = utils.MatchCRS.geometry(geom, geo_crs, self.crs)
+            return utils.ESRIGeomQuery((geom.x, geom.y), self.out_sr).point()
+
+        if isinstance(geom, MultiPoint):
+            geom = utils.MatchCRS.geometry(geom, geo_crs, self.crs)
+            return utils.ESRIGeomQuery([(g.x, g.y) for g in geom], self.out_sr).multipoint()
+
+        if isinstance(geom, Polygon):
+            geom = utils.MatchCRS.geometry(geom, geo_crs, self.crs)
+            return utils.ESRIGeomQuery(geom, self.out_sr).polygon()
+
+        raise InvalidInputType("geom", "Polygon, Point, MultiPoint, tuple, or list of tuples")
+
     def __repr__(self) -> str:
         """Print the service configuration."""
         extent = ", ".join(f"{c:.3f}" for c in self.extent) if self.extent else None
+        ftypes = ", ".join(self.feature_types.values()) if self.feature_types else None
         return "\n".join(
             [
                 "Service configurations:",
-                f"URL: {self.base_url}",
-                f"Max Record Count: {self.max_nrecords}",
-                f"Supported Query Formats: {self.query_formats}",
-                f"Units: {self.units}",
-                f"Extent: ({extent})",
+                f"    URL: {self.base_url}",
+                f"    Max Record Count: {self.max_nrecords}",
+                f"    Supported Query Formats: {self.query_formats}",
+                f"    Units: {self.units}",
+                f"    Extent: ({extent})",
+                f"    Feature Types: {ftypes}",
             ]
         )
 
 
-@dataclass
 class WMSBase:
     """Base class for accessing a WMS service.
 
@@ -262,11 +331,19 @@ class WMSBase:
         epsg:4326.
     """
 
-    url: str
-    layers: Union[str, List[str]]
-    outformat: str
-    version: str = "1.3.0"
-    crs: str = DEF_CRS
+    def __init__(
+        self,
+        url: str,
+        layers: Union[str, List[str]],
+        outformat: str,
+        version: str = "1.3.0",
+        crs: str = DEF_CRS,
+    ) -> None:
+        self.url = url
+        self.layers = layers
+        self.outformat = outformat
+        self.version = version
+        self.crs = crs
 
     def __repr__(self) -> str:
         """Print the services properties."""
@@ -308,7 +385,6 @@ class WMSBase:
         return {wms[lyr].name: wms[lyr].title for lyr in list(wms.contents)}
 
 
-@dataclass
 class WFSBase:
     """Base class for WFS service.
 
@@ -331,11 +407,19 @@ class WFSBase:
         epsg:4326.
     """
 
-    url: str
-    layer: Optional[str] = None
-    outformat: Optional[str] = None
-    version: str = "2.0.0"
-    crs: str = DEF_CRS
+    def __init__(
+        self,
+        url: str,
+        layer: Optional[str] = None,
+        outformat: Optional[str] = None,
+        version: str = "2.0.0",
+        crs: str = DEF_CRS,
+    ) -> None:
+        self.url = url
+        self.layer = layer
+        self.outformat = outformat
+        self.version = version
+        self.crs = crs
 
     def __repr__(self) -> str:
         """Print the services properties."""
