@@ -58,7 +58,7 @@ class ArcGISRESTful(ArcGISRESTfulBase):
             Tuple[float, float, float, float],
         ],
         geo_crs: str = DEF_CRS,
-        sql_clause: str = "",
+        sql_clause: Optional[str] = None,
         distance: Optional[int] = None,
     ) -> None:
         """Get feature IDs within a geometry that can be combined with a SQL where clause.
@@ -71,34 +71,16 @@ class ArcGISRESTful(ArcGISRESTfulBase):
         geo_crs : str
             The spatial reference of the input geometry, defaults to EPSG:4326.
         sql_clause : str, optional
-            A valid SQL 92 WHERE clause, default to an empty string.
+            A valid SQL 92 WHERE clause, default to None.
         distance : int, optional
             The buffer distance for the input geometries in meters, default to None.
         """
-        if not isinstance(sql_clause, str):
-            raise InvalidInputType("sql_clause", str)
-
         if isinstance(geom, tuple) and len(geom) == 2:
             geom = Point(geom)
         elif isinstance(geom, list) and all(len(g) == 2 for g in geom):
             geom = MultiPoint(geom)
 
-        geom_query = None
-        if isinstance(geom, tuple) and len(geom) == 4:
-            geom = MatchCRS.bounds(geom, geo_crs, self.crs)  # type: ignore
-            geom_query = utils.ESRIGeomQuery(geom, self.out_sr).bbox()
-        elif isinstance(geom, Point):
-            geom = MatchCRS.geometry(geom, geo_crs, self.crs)
-            geom_query = utils.ESRIGeomQuery((geom.x, geom.y), self.out_sr).point()
-        elif isinstance(geom, MultiPoint):
-            geom = MatchCRS.geometry(geom, geo_crs, self.crs)
-            geom_query = utils.ESRIGeomQuery([(g.x, g.y) for g in geom], self.out_sr).multipoint()
-        elif isinstance(geom, Polygon):
-            geom = MatchCRS.geometry(geom, geo_crs, self.crs)
-            geom_query = utils.ESRIGeomQuery(geom, self.out_sr).polygon()
-
-        if geom_query is None:
-            raise InvalidInputType("geom", "Polygon, Point, MultiPoint, tuple, or list of tuples")
+        geom_query = self._esri_query(geom, geo_crs)
 
         payload = {
             **geom_query,  # type: ignore
@@ -108,19 +90,15 @@ class ArcGISRESTful(ArcGISRESTfulBase):
             "f": self.outformat,
         }
         if distance:
-            payload["distance"] = f"{distance}"
-            payload["units"] = "esriSRUnit_Meter"
+            payload.update({"distance": f"{distance}", "units": "esriSRUnit_Meter"})
 
-        if len(sql_clause) > 0:
+        if sql_clause:
             payload.update({"where": sql_clause})
 
         resp = self.session.post(f"{self.base_url}/query", payload)
 
         try:
-            r_json = resp.json()["objectIds"]
-            if r_json is None:
-                raise ZeroMatched(self._zeromatched)
-            self.featureids = r_json
+            self.featureids = resp.json()["objectIds"]
         except (KeyError, TypeError, IndexError, JSONDecodeError):
             raise ZeroMatched(self._zeromatched)
 
@@ -165,7 +143,7 @@ class ArcGISRESTful(ArcGISRESTfulBase):
             A valid SQL 92 WHERE clause.
         """
         if not isinstance(sql_clause, str):
-            raise InvalidInputType("sql_clause", str)
+            raise InvalidInputType("sql_clause", "str")
 
         payload = {
             "where": sql_clause,
@@ -613,16 +591,20 @@ class ServiceURL:
 
     @property
     def restful(self) -> SimpleNamespace:
+        """Read RESTful URLs from the source yml file."""
         return self._make_nt("restful")
 
     @property
     def wms(self) -> SimpleNamespace:
+        """Read WMS URLs from the source yml file."""
         return self._make_nt("wms")
 
     @property
     def wfs(self) -> SimpleNamespace:
+        """Read WFS URLs from the source yml file."""
         return self._make_nt("wfs")
 
     @property
     def http(self) -> SimpleNamespace:
+        """Read HTTP URLs from the source yml file."""
         return self._make_nt("http")
