@@ -1,8 +1,8 @@
 """Some utilities for PyGeoOGC."""
 import asyncio
+import math
 import socket
 from concurrent import futures
-from dataclasses import dataclass
 from typing import (
     Any,
     Callable,
@@ -19,11 +19,10 @@ from unittest.mock import _patch, patch
 
 import aiohttp
 import cytoolz as tlz
+import defusedxml.ElementTree as etree
 import nest_asyncio
-import numpy as np
 import orjson as json
 import pyproj
-from defusedxml import cElementTree as etree
 from requests import Response, Session
 from requests.adapters import HTTPAdapter
 from requests.exceptions import RequestException
@@ -164,7 +163,7 @@ async def _request_binary(
         The retrieved response as binary
     """
     async with session_req(url, **payload) as response:
-        return await response.read()  # type: ignore
+        return await response.read()
 
 
 async def _request_json(
@@ -189,7 +188,7 @@ async def _request_json(
         The retrieved response as json
     """
     async with session_req(url, **payload) as response:
-        return await response.json()  # type: ignore
+        return await response.json()
 
 
 async def _request_text(
@@ -214,7 +213,7 @@ async def _request_text(
         The retrieved response as string
     """
     async with session_req(url, **payload) as response:
-        return await response.text()  # type: ignore
+        return await response.text()
 
 
 async def _async_session(
@@ -285,10 +284,10 @@ def async_requests(
     """
     chunked_urls = tlz.partition_all(max_workers, url_payload)
 
-    results: List[Union[str, MutableMapping[str, Any], bytes]] = []
-    for chunk in chunked_urls:
-        loop = asyncio.get_event_loop()
-        results.append(loop.run_until_complete(_async_session(chunk, read, request)))  # type: ignore
+    results = (
+        asyncio.get_event_loop().run_until_complete(_async_session(c, read, request))
+        for c in chunked_urls
+    )
     return list(tlz.concat(results))
 
 
@@ -362,7 +361,7 @@ def traverse_json(
         key = path[ind]
         if ind + 1 < len(path):
             if isinstance(obj, dict):
-                if key in obj.keys():
+                if key in obj:
                     extract(obj.get(key), path, ind + 1, arr)
                 else:
                     arr.append(None)
@@ -380,9 +379,9 @@ def traverse_json(
                     arr.append(None)
                 else:
                     for item in obj:
-                        arr.append(item.get(key, None))
+                        arr.append(item.get(key))
             elif isinstance(obj, dict):
-                arr.append(obj.get(key, None))
+                arr.append(obj.get(key))
             else:
                 arr.append(None)
         return arr
@@ -398,7 +397,6 @@ def traverse_json(
     return []
 
 
-@dataclass
 class ESRIGeomQuery:
     """Generate input geometry query for ArcGIS RESTful services.
 
@@ -414,14 +412,22 @@ class ESRIGeomQuery:
         for reference.
     """
 
-    geometry: Union[
-        Tuple[float, float], List[Tuple[float, float]], Tuple[float, float, float, float], Polygon
-    ]
-    wkid: int
+    def __init__(
+        self,
+        geometry: Union[
+            Tuple[float, float],
+            List[Tuple[float, float]],
+            Tuple[float, float, float, float],
+            Polygon,
+        ],
+        wkid: int,
+    ) -> None:
+        self.geometry = geometry
+        self.wkid = wkid
 
     def point(self) -> Dict[str, Union[str, bytes]]:
         """Query for a point."""
-        if not isinstance(self.geometry, tuple) and len(self.geometry) != 2:
+        if not (isinstance(self.geometry, tuple) and len(self.geometry) == 2):
             raise InvalidInputType("geometry (point)", "tuple", "(x, y)")
 
         geo_type = "esriGeometryPoint"
@@ -430,16 +436,16 @@ class ESRIGeomQuery:
 
     def multipoint(self) -> Dict[str, Union[str, bytes]]:
         """Query for a multi-point."""
-        if not isinstance(self.geometry, list) and any(len(g) != 2 for g in self.geometry):  # type: ignore
+        if not (isinstance(self.geometry, list) and all(len(g) == 2 for g in self.geometry)):
             raise InvalidInputType("geometry (multi-point)", "list of tuples", "[(x, y), ...]")
 
         geo_type = "esriGeometryMultipoint"
-        geo_json = {"points": [[x, y] for x, y in self.geometry]}  # type: ignore
+        geo_json = {"points": [[x, y] for x, y in self.geometry]}
         return self.get_payload(geo_type, geo_json)
 
     def bbox(self) -> Dict[str, Union[str, bytes]]:
         """Query for a bbox."""
-        if not isinstance(self.geometry, (tuple, list)) and len(self.geometry) != 4:
+        if not (isinstance(self.geometry, (tuple, list)) and len(self.geometry) == 4):
             raise InvalidInputType("geometry (bbox)", "tuple or list", BOX_ORD)
 
         geo_type = "esriGeometryEnvelope"
@@ -456,6 +462,15 @@ class ESRIGeomQuery:
         return self.get_payload(geo_type, geo_json)
 
     def get_payload(self, geo_type: str, geo_json: Dict[str, Any]) -> Dict[str, Union[str, bytes]]:
+        """Generate a request payload based on ESRI template.
+
+        Parameters
+        ----------
+        geo_type : str
+            Type of the input geometry
+        geo_json : dict
+            Geometry in GeoJson format.
+        """
         esri_json = json.dumps({**geo_json, "spatialRelference": {"wkid": str(self.wkid)}})
         return {
             "geometryType": geo_type,
@@ -481,6 +496,7 @@ class MatchCRS:
     def geometry(
         geom: Union[Polygon, MultiPolygon, Point, MultiPoint], in_crs: str, out_crs: str
     ) -> Union[Polygon, MultiPolygon, Point, MultiPoint]:
+        """Reproject a geometry to the specified output CRS."""
         if not isinstance(geom, (Polygon, MultiPolygon, Point, MultiPoint)):
             raise InvalidInputType("geom", "Polygon, MultiPolygon, Point, or MultiPoint")
 
@@ -491,7 +507,8 @@ class MatchCRS:
     def bounds(
         geom: Tuple[float, float, float, float], in_crs: str, out_crs: str
     ) -> Tuple[float, float, float, float]:
-        if not isinstance(geom, tuple) and len(geom) != 4:
+        """Reproject a bounding box to the specified output CRS."""
+        if not (isinstance(geom, tuple) and len(geom) == 4):
             raise InvalidInputType("geom", "tuple of length 4", BOX_ORD)
 
         project = pyproj.Transformer.from_crs(in_crs, out_crs, always_xy=True).transform
@@ -502,7 +519,8 @@ class MatchCRS:
     def coords(
         geom: Tuple[Tuple[float, ...], Tuple[float, ...]], in_crs: str, out_crs: str
     ) -> Tuple[Any, ...]:
-        if not isinstance(geom, tuple) and len(geom) != 2:
+        """Reproject a list of coordinates to the specified output CRS."""
+        if not (isinstance(geom, tuple) and len(geom) == 2):
             raise InvalidInputType("geom", "tuple of length 2", "((xs), (ys))")
 
         project = pyproj.Transformer.from_crs(in_crs, out_crs, always_xy=True).transform
@@ -596,7 +614,7 @@ def bbox_decompose(
             return (dst, lvl, az, dx, 0) if xy else (lvl, dst, az, dx, 1)
 
         while divs[-1] < 1:
-            dim = int(np.sqrt(max_px) * mul)
+            dim = int(math.sqrt(max_px) * mul)
             step = (dim - 1) * resolution
 
             _dest = origin
